"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _joi = _interopRequireDefault(require("joi"));

var _get = _interopRequireDefault(require("lodash/get"));

var _flatten = _interopRequireDefault(require("lodash/flatten"));

var _oors = require("oors");

var _PermissionsManager = _interopRequireDefault(require("./libs/PermissionsManager"));

var schemas = _interopRequireWildcard(require("./schemas"));

var _Security = _interopRequireDefault(require("./services/Security"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class Security extends _oors.Module {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "name", 'oors.security');

    _defineProperty(this, "hooks", {
      'oors.security.createPermissionsExtractor': ({
        security
      }) => user => security.getUserPermissions({
        userId: user._id
      })
    });
  }

  // eslint-disable-next-line class-methods-use-this
  buildExtractor(extractors) {
    return async user => {
      const permissionsLists = await Promise.all(extractors.filter(extractor => extractor).map(extractor => extractor(user)));
      return (0, _flatten.default)(permissionsLists);
    };
  }

  async setup() {
    const [{
      createRepository
    }, {
      registerServices
    }] = await this.dependencies(['oors.mongodb', 'oors.octobus']);
    const services = registerServices(this, {
      GroupRepository: createRepository({
        name: 'SecurityGroup',
        schema: schemas.group
      }),
      UserRepository: createRepository({
        name: 'SecurityUser',
        schema: schemas.user
      }),
      Security: new _Security.default()
    });
    const extractors = await this.runHook('createPermissionsExtractor', // eslint-disable-next-line consistent-return
    module => {
      const permissionsExtractor = (0, _get.default)(module, 'security.permissionsExtractor');

      if (permissionsExtractor) {
        return permissionsExtractor({
          security: services.Security
        });
      }
    }, {
      security: services.Security
    });
    const permissionsManager = new _PermissionsManager.default({
      extractor: this.buildExtractor(extractors)
    });
    await this.runHook('definePermissions', module => {
      const modulePermissions = (0, _get.default)(module, 'security.permissions');

      if (modulePermissions) {
        Object.keys(modulePermissions).forEach(permission => {
          permissionsManager.define(permission, modulePermissions[permission]);
        });
      }
    }, {
      permissionsManager
    });
    this.export(_objectSpread({}, services, {
      permissionsManager
    }));
  }

}

_defineProperty(Security, "schema", {
  extractUserPermissions: _joi.default.func()
});

var _default = Security;
exports.default = _default;